# sb::App::Xmlrpc
# == Author(s) : Takuya Otani <takuya.otani@gmail.com> ==
# == Copyright (C) 2005 SimpleBoxes/SerendipityNZ Ltd. ==

# ======================================
package sb::App::Xmlrpc;
use strict;
use sb::Plugin ();
use sb::Data ();
use sb::Text ();
use sb::Time ();
use sb::Template ();
use sb::Lock ();
use sb::Utils ();
use sb::Admin::Entry ();
use sb::App::Admin ();
use sb::App ();
use base qw( sb::App );
use vars qw( $VERSION );
$VERSION = '0.05';
# 0.05 [2007/12/01] ported from Serene Bach 2
# 0.04 [2007/07/20] fixed a bug in value
# 0.03 [2007/07/16] changed return to make them consistent
# 0.02 [2007/07/08] changed response tu use new sb::Template
# 0.01 [2006/03/17] changed bracket rule
# 0.00 [2006/03/04] refactored
# --------------------------------------
# // defines / static variables
sub OUTPUT_CHARSET  (){ 'utf-8' };
sub OUTPUT_CHARCODE (){ 'utf8' };
sub CONTENT_TYPE    (){ 'text/xml' };
sub PREFIX_NAME     (){ 'file' }; # file prefix
sub TIME_FORMAT     (){ '%Year%%Mon%%Day%T%Hour%:%Min%:%Sec%' };
sub TEXT_FORMAT_1   (){ 'Auto line breaking' };
my @mScriptsForEntries = (
	'latest_entry_list','category_list','archives_list','calendar','calendar2','calendar_vertical','calendar_horizontal',
);
# --------------------------------------
# // override base class
sub initialize
{
	my $self = shift;
	$self->SUPER::initialize(
		'methods' => {
			'blogger.newPost'           => \&_blogger_newpost,
			'blogger.editPost'          => \&_blogger_editpost,
			'blogger.getRecentPosts'    => \&_blogger_getrecentposts,
			'blogger.getUsersBlogs'     => \&_blogger_getusersblogs,
			'blogger.getUserInfo'       => \&_blogger_getuserinfo,
			'blogger.deletePost'        => \&_blogger_deletepost,
			'metaWeblog.getPost'        => \&_metaweblog_getpost,
			'metaWeblog.newPost'        => \&_metaweblog_newpost,
			'metaWeblog.editPost'       => \&_metaweblog_editpost,
			'metaWeblog.getRecentPosts' => \&_metaweblog_getrecentposts,
			'metaWeblog.newMediaObject' => \&_metaweblog_newmediaobject,
			'mt.getCategoryList'        => \&_mt_getcategorylist,
			'mt.setPostCategories'      => \&_mt_setpostcategories,
			'mt.getPostCategories'      => \&_mt_getpostcategories,
			'mt.getRecentPostTitles'    => \&_mt_recentposttitles,
			'mt.publishPost'            => \&_mt_publishpost,
			'mt.supportedMethods'       => \&_mt_supportedmethods,
			'mt.supportedTextFilters'   => \&_mt_supportedtextfilters,
			'sb.getMediaObjectList'     => \&_getmediaobjectlist,
			'sb.getMediaObject'         => \&_getmediaobject,
			'sb.deleteMediaObject'      => \&_deletemediaobject,
		},
		'user'    => undef,
		'users'   => {sb::Data->load_as_hash('User')},
		'blog'    => undef,
		'cat'     => undef,
		'ents'    => undef,
		'lock'    => undef,
		'charset' => OUTPUT_CHARSET,
		@_
	);
	$self->generate_accessor('methods','user','users','charset');
	$self->charset($self->cgi->value('charset')) if ($self->cgi->value('charset'));
	return $self;
}
# --------------------------------------
# // override base class - sb::App
sub run
{
	my $class = shift;
	my $self = $class->new(@_);
	if (!$self->cgi->xmlrpc)
	{
		$self->error(sb->string('Unknown error.'));
		return;
	}
	my $error = undef;
	my $method = $self->methods->{$self->_method};
	if (! defined($method) or ref($method) ne 'CODE')
	{
		$self->response(1,sb::Text->new($self->_method)->entitized);
	}
	else
	{
		eval{ $self->response($self->$method($self->_params)); };
		$self->response(99,sb::Text->new($@)->entitized) if ($@);
	}
}
# --------------------------------------
# // public functions
sub response
{
	my $self = shift;
	my ($err,$msg) = @_;
	my $temp = sb::Template->new($self->_template($err));
	$msg = $self->error_code($err,$msg) if ($err);
	$temp->num(0);
	$temp->tag('site_encoding'=>OUTPUT_CHARSET);
	$temp->tag('xmlrpc_error'=>$err);
	$temp->tag('xmlrpc_message'=>$msg);
	my $output = $temp->output;
	if ($self->lang->charcode ne OUTPUT_CHARCODE)
	{
		$output = $self->lang->convert($output,OUTPUT_CHARCODE,$self->lang->charcode);
	}
	$self->cgi->head(
		'type'    => CONTENT_TYPE,
		'charset' => OUTPUT_CHARSET,
		'length'  => length($output),
	);
	$self->output($output);
	$self->{'lock'}->unlock if ($self->{'lock'});
}
sub lock
{
	my $self = shift;
	$self->{'lock'} = sb::Lock->lock if (!$self->{'lock'});
	return $self->{'lock'};
}
sub cat
{
	my $self = shift;
	return $self->{'cat'} if ($self->{'cat'});
	$self->{'cat'} = {sb::Data->load_as_hash('Category')};
	return $self->{'cat'};
}
sub ents
{
	my $self = shift;
	return $self->{'ents'} if ($self->{'ents'});
	$self->{'ents'} = [sb::Data->load('Entry','sort'=>'date','order'=>0)];
	return $self->{'ents'};
}
sub blog
{
	my $self = shift;
	return $self->{'blog'} if ($self->{'blog'});
	$self->{'blog'} = sb::Data->load('Weblog','id'=>0);
	return $self->{'blog'};
}
sub error_code
{
	my $self = shift;
	my ($err,$msg) = @_;
	my %errs = (
		0  => 'No error.',                  # works fine
		1  => 'Unknown method. [%s]',       # it's not supported
		2  => 'Too many parameters.',       # parameters more than requested
		3  => 'Failed to authorize.',       # could be wrong password
		4  => 'The object does not exist.', # no target data
		5  => 'Failed to lock a file.',     # a file could be still locked
		6  => 'Failed to upload a file.',   # uploading does not succeed
		7  => 'Permission denied.',         # it's not permitted
		8  => 'Lack of parameters.',        # parameters less than requested
		99 => 'Unknown error. %s',          # something wrong
	);
	return sprintf($errs{$err},$msg);
}
sub check_login
{
	my $self = shift;
	my %param = (
		'user' => undef,
		'pass' => undef,
		@_
	);
	return if (!$self->users);
	return if ($param{'user'} eq '' or $param{'pass'} eq '');
	foreach my $id ( keys( %{$self->users} ) )
	{
		my $user = $self->users->{$id};
		next if ( $user->name ne $param{'user'} );
		return if (!$user->check_pass($param{'pass'}));
		$self->user($user);
		last;
	}
	return 1;
}
sub has_permission
{
	return sb::App::Admin::has_permission(@_);
}
sub _get_neighbor
{
	my $self = shift;
	$self->ents();
	return sb::Admin::Entry::_get_neighbor($self,@_);
}
# --------------------------------------
# // private functions - xmlrpc methods
sub _blogger_newpost
{
	my $self = shift;
	my $app  = shift; # string (not use)
	my $wid  = shift; # string (not use)
	my $user = _value(shift); # string
	my $pass = _value(shift); # string
	my $body = _value(shift); # string
	my $flag = _value(shift); # boolean
	return (2,undef) if (@_);
	return (3,undef) if (!$self->check_login('user'=>$user,'pass'=>$pass));
	return (5,undef) if (!$self->lock);
	my $entry = $self->_edit_entry(
		'entry'   => undef,
		'content' => {'description' => $body},
		'open'    => $flag,
	);
	return (0,($entry) ? '<string>' . $entry->id . '</string>' : '<boolean>0</boolean>');
}
sub _blogger_editpost
{
	my $self = shift;
	my $app  = shift;  # string (not use)
	my $eid  = _value(shift); # string
	my $user = _value(shift); # string
	my $pass = _value(shift); # string
	my $body = _value(shift); # string
	my $flag = _value(shift); # boolean
	return (2,undef) if (@_);
	return (3,undef) if (!$self->check_login('user'=>$user,'pass'=>$pass));
	return (5,undef) if (!$self->lock);
	my $entry = sb::Data->load('Entry','id'=>$eid);
	return (4,undef) if (!$entry);
	return (7,undef) if (!$self->has_permission('id'=>$entry->auth));
	$entry = $self->_edit_entry(
		'entry'   => $entry,
		'content' => {'description' => $body},
		'open'    => $flag,
	);
	return (0,($entry) ? '<boolean>1</boolean>' : '<boolean>0</boolean>');
}
sub _blogger_getrecentposts
{
	my $self = shift;
	my $app  = shift; # string (not use)
	my $wid  = shift; # string (not use)
	my $user = _value(shift); # string
	my $pass = _value(shift); # string
	my $num  = _value(shift); # int
	return (2,undef) if (@_);
	return (3,undef) if (!$self->check_login('user'=>$user,'pass'=>$pass));
	my @entries = sb::Data->load('Entry',
		'bgn'    => 0,
		'num'    => $num,
		'sort'   => 'date',
		'order'  => 1,
	);
	my $out = '<array><data>' . "\n";
	foreach my $entry (@entries) {
		$out .= '<value>' . $self->_entry_struct('entry'=>$entry,'mode'=>'simple') . '</value>';
	}
	$out .= '</data></array>';
	return (0,$out);
}
sub _blogger_getusersblogs
{
	my $self = shift;
	my $app  = shift; # string (not use)
	my $user = _value(shift); # string
	my $pass = _value(shift); # string
	return (2,undef) if (@_);
	return (3,undef) if (!$self->check_login('user'=>$user,'pass'=>$pass));
	my @out = ('<array><data><value><struct>');
	push(@out,'<member><name>url</name><value><string>' . $self->conf->value('conf_srv_base') . '</string></value></member>');
	push(@out,'<member><name>blogid</name><value><string>0</string></value></member>');
	push(@out,'<member><name>blogName</name><value><string>' . $self->blog->title . '</string></value></member>');
	push(@out,'</struct></value></data></array>');
	return (0,join("\n",@out));
}
sub _blogger_getuserinfo
{
	my $self = shift;
	my $app  = shift; # string (not use)
	my $user = _value(shift); # string
	my $pass = _value(shift); # string
	return (2,undef) if (@_);
	return (3,undef) if (!$self->check_login('user'=>$user,'pass'=>$pass));
	$user = $self->{'user'};
	my $name = $user->real;
	my @out = ('<struct>');
	push(@out,'<member><name>userid</name><value><string>' . $user->id . '</string></value></member>');
	push(@out,'<member><name>firstname</name><value><string>' . (split(/\s+/,$name,2))[0] . '</string></value></member>');
	push(@out,'<member><name>lastname</name><value><string>' . (split(/\s+/,$name,2))[1] . '</string></value></member>');
	push(@out,'<member><name>nickname</name><value><string>' . $name . '</string></value></member>');
	push(@out,'<member><name>email</name><value><string>' . $user->mail . '</string></value></member>');
	push(@out,'<member><name>url</name><value><string>' . $self->conf->value('conf_srv_base') . '</string></value></member>');
	push(@out,'</struct>');
	return (0,join("\n",@out));
}
sub _blogger_deletepost
{
	my $self = shift;
	my $app  = shift; # string (not use)
	my $eid  = _value(shift); # string
	my $user = _value(shift); # string
	my $pass = _value(shift); # string
	my $flag = shift; # boolean (not use)
	return (2,undef) if (@_);
	return (3,undef) if (!$self->check_login('user'=>$user,'pass'=>$pass));
	return (5,undef) if (!$self->lock);
	my $entry = sb::Data->load('Entry','id'=>$eid);
	return (4,undef) if (!$entry);
	return (7,undef) if (!$self->has_permission('id'=>$entry->auth));
	$entry->erase;
	sb::Data->update($entry);
	return (0,'<boolean>1</boolean>');
}
sub _metaweblog_getpost
{
	my $self = shift;
	my $eid  = _value(shift); # string
	my $user = _value(shift); # string
	my $pass = _value(shift); # string
	return (2,undef) if (@_);
	return (3,undef) if (!$self->check_login('user'=>$user,'pass'=>$pass));
	my $entry = sb::Data->load('Entry','id'=>$eid);
	return (4,undef) if (!$entry);
	return (0,$self->_entry_struct('entry'=>$entry,'mode'=>'detail'));
}
sub _metaweblog_newpost
{
	my $self = shift;
	my $wid  = shift; # string (not use)
	my $user = _value(shift); # string
	my $pass = _value(shift); # string
	my $cont = _value(shift); # struct
	my $flag = _value(shift); # boolean
	return (2,undef) if (@_);
	return (3,undef) if (!$self->check_login('user'=>$user,'pass'=>$pass));
	return (5,undef) if (!$self->lock);
	my $entry = $self->_edit_entry(
		'entry'   => undef,
		'content' => $cont,
		'open'    => $flag,
	);
	return (0,($entry) ? '<string>' . $entry->id . '</string>' : '<boolean>0</boolean>');
}
sub _metaweblog_editpost
{
	my $self = shift;
	my $eid  = _value(shift); # string
	my $user = _value(shift); # string
	my $pass = _value(shift); # string
	my $cont = _value(shift); # struct
	my $flag = _value(shift); # boolean
	return (2,undef) if (@_);
	return (3,undef) if (!$self->check_login('user'=>$user,'pass'=>$pass));
	return (5,undef) if (!$self->lock);
	my $entry = sb::Data->load('Entry','id'=>$eid);
	return (4,undef) if (!$entry);
	return (7,undef) if (!$self->has_permission('id'=>$entry->auth));
	$entry = $self->_edit_entry(
		'entry'   => $entry,
		'content' => $cont,
		'open'    => $flag,
	);
	return (0,($entry) ? '<boolean>1</boolean>' : '<boolean>0</boolean>');
}
sub _metaweblog_getrecentposts
{
	my $self = shift;
	my $wid  = shift; # string (not use)
	my $user = _value(shift); # string
	my $pass = _value(shift); # string
	my $num  = _value(shift); # int
	return (2,undef) if (@_);
	return (3,undef) if (!$self->check_login('user'=>$user,'pass'=>$pass));
	my @entries = sb::Data->load('Entry',
		'bgn'    => 0,
		'num'    => $num,
		'sort'   => 'date',
		'order'  => 1,
	);
	my $out = '<array><data>' . "\n";
	foreach my $entry (@entries)
	{
		$out .= '<value>' . $self->_entry_struct('entry'=>$entry,'mode'=>'detail') . '</value>';
	}
	$out .= '</data></array>';
	return (0,$out);
}
sub _metaweblog_newmediaobject
{
	my $self = shift;
	my $wid  = shift; # string (not use)
	my $user = _value(shift); # string
	my $pass = _value(shift); # string
	my $file = _value(shift); # struct
	return (2,undef) if (@_);
	return (3,undef) if (!$self->check_login('user'=>$user,'pass'=>$pass));
	return (5,undef) if (!$self->lock);
	return (8,undef) if (!$file or $file->{'name'} eq '');
	my $name  = $file->{'name'};
	if ($self->lang->charset ne $self->charset)
	{
		my $charcode = $self->lang->code_for_charset($self->charset);
		$name = $self->lang->convert($name,$self->lang->charcode,$charcode);
	}
	my $field = 'upload_file:';
	$field .= ' Content-Type:' . $file->{'type'}  . ';' if ($file->{'type'} ne '');
	$field .= ' filename=' . $name;
	my $loader = sb::File::Uploader->new();
	$loader->path($self->conf->value('conf_dir_base') . $self->conf->value('conf_dir_img')); # set dir to upload
	$loader->set_param( # prepare to upload
		'label'  => 'upload_file',
		'header' => [$field]
	);
	if ($loader->uploaded_name ne '' and $loader->uploaded_extension ne '')
	{
		my $filenum = sb::App::Admin->_get_latest_imageid();
		if ( defined($filenum) and !$name )
		{ # put the prefix to the filename
			my $filename  = $loader->uploaded_name;
			if ($filename =~ /[^a-zA-Z0-9_\-\.]/)
			{
				$filename = sb::Text->new($filename)->format('md5','encode');
				$filename = substr($filename,0,64) if (length($filename) > 64);
				$filename .= $loader->uploaded_extension;
			}
			$loader->name(PREFIX_NAME . ++$filenum . '_' . $filename); # set name to upload
		}
		else
		{ # keep the name of file to upload
			$loader->name($loader->uploaded_name); # set name to upload
		}
		if ( !$loader->exists and $loader->upload($file->{'bits'}) )
		{
			my $new_file = sb::Data->add('Image',
				'auth' => $self->user->id,
				'name' => $loader->{'name'},
				'date' => $self->time->data,
				'tz'   => $self->conf->value('conf_timezone'),
			);
			if ($new_file)
			{
				$new_file->set_file($loader->{'file'},$self->conf->value('conf_thumbcheck'));
				sb::Data->update($new_file);
				my @out = ();
				push(@out,'<struct>');
				push(@out,'<member><name>url</name><value><string>' . $new_file->get_url . '</string></value></member>');
				push(@out,'</struct>');
				return (0,join("\n",@out));
			} # end of if ($new_file)
		} # end of if (!$loader->exists and $loader->upload($file->{'bits'}))
	} # end of if ( $loader->uploaded_name ne '' and $loader->uploaded_extension ne '')
	return (6,undef);
}
sub _mt_getcategorylist
{
	my $self = shift;
	my $wid  = shift; # string (not use)
	my $user = _value(shift); # string
	my $pass = _value(shift); # string
	return (2,undef) if (@_);
	return (3,undef) if (!$self->check_login('user'=>$user,'pass'=>$pass));
	my $list = $self->_sort_categories('cat'=>[ sort { $b->order <=> $a->order } values(%{$self->cat}) ]);
	my @out = ('<array><data>');
	foreach my $id ( split(',',$list) )
	{
		next if ($id eq '');
		my $cat = $self->cat->{$id};
		my $name = $cat->fullname($self->cat);
		push(@out,'<value><struct>');
		push(@out,'<member><name>categoryId</name><value><string>' . $cat->id . '</string></value></member>');
		push(@out,'<member><name>categoryName</name><value><string>' . $name . '</string></value></member>');
		push(@out,'</struct></value>');
	}
	push(@out,'</data></array>');
	return (0,join("\n",@out));
}
sub _mt_setpostcategories
{
	my $self = shift;
	my $eid  = _value(shift); # string
	my $user = _value(shift); # string
	my $pass = _value(shift); # string
	my $cats = _value(shift); # array [ struct ]
	return (2,undef) if (@_);
	return (3,undef) if (!$self->check_login('user'=>$user,'pass'=>$pass));
	return (5,undef) if (!$self->lock);
	return (8,undef) if (!$cats);
	my $entry = sb::Data->load('Entry','id'=>$eid);
	return (4,undef) if (!$entry);
	return (7,undef) if (!$self->has_permission('id'=>$entry->auth));
	my $primary = '';
	my @related = ();
	foreach my $cat (@{$cats})
	{
		my $id = int($cat->{'categoryId'});
		next if (!$self->cat->{$id});
		if ($cat->{'isPrimary'} and $primary eq '')
		{
			$primary = $id;
		}
		else
		{
			push(@related,$id);
		}
	}
	$entry->cat($primary);
	$entry->add(',' . join(',',@related) . ',') if (@related);
	$entry->edit($self->{'user'}->id);
	sb::Data->update($entry);
	return (0,'<boolean>1</boolean>');
}
sub _mt_getpostcategories
{
	my $self = shift;
	my $eid  = _value(shift); # string
	my $user = _value(shift); # string
	my $pass = _value(shift); # string
	return (2,undef) if (@_);
	return (3,undef) if (!$self->check_login('user'=>$user,'pass'=>$pass));
	my $entry = sb::Data->load('Entry','id'=>$eid);
	return (4,undef) if (!$entry);
	return (7,undef) if (!$self->has_permission('id'=>$entry->auth));
	my @out  = ('<array><data>');
	my @cats = ($entry->cat);
	push(@cats,split(',',$entry->add)) if ($entry->add ne '');
	foreach my $id (@cats)
	{
		next if ($id eq '');
		my $cat = $self->cat->{$id};
		if ($cat)
		{
			my $name = $cat->fullname($self->cat);
			push(@out,'<value><struct>');
			push(@out,'<member><name>categoryId</name><value><string>' . $cat->id . '</string></value></member>');
			push(@out,'<member><name>categoryName</name><value><string>' . $name . '</string></value></member>');
			if ($entry->cat ne '' and $cat->id eq $entry->cat)
			{
				push(@out,'<member><name>isPrimary</name><value><boolean>1</boolean></value></member>');
			}
			push(@out,'</struct></value>');
		}
	}
	push(@out,'</data></array>');
	return (0,join("\n",@out));
}
sub _mt_recentposttitles
{
	my $self = shift;
	my $wid  = shift; # string (not use)
	my $user = _value(shift); # string
	my $pass = _value(shift); # string
	my $num  = _value(shift); # int
	return (2,undef) if (@_);
	return (3,undef) if (!$self->check_login('user'=>$user,'pass'=>$pass));
	my @entries = sb::Data->load('Entry',
		'bgn'    => 0,
		'num'    => $num,
		'sort'   => 'date',
		'order'  => 1,
	);
	my $out = '<array><data>' . "\n";
	map { $out .= '<value>' . $self->_entry_struct('entry'=>$_,'mode'=>'basic') . '</value>' } @entries;
	$out .= '</data></array>';
	return (0,$out);
}
sub _mt_publishpost
{
	my $self = shift;
	my $eid  = _value(shift); # string
	my $user = _value(shift); # string
	my $pass = _value(shift); # string
	return (2,undef) if (@_);
	return (3,undef) if (!$self->check_login('user'=>$user,'pass'=>$pass));
	return (5,undef) if (!$self->lock);
	my $entry = sb::Data->load('Entry','id'=>$eid);
	return (4,undef) if (!$entry);
	return (7,undef) if (!$self->has_permission('id'=>$entry->auth));
	$self->_build_files($entry);
	return (0,'<boolean>1</boolean>');
}
sub _mt_supportedmethods
{
	my $self = shift;
	return (2,undef) if (@_);
	my $out = '<array><data>' . "\n";
	foreach my $name ( keys(%{$self->methods}) )
	{
		next if (ref($self->methods->{$name}) ne 'CODE');
		next if ($name eq 'mt.supportedMethods');
		$out .= '<value><string>' . $name . '</string></value>' . "\n";
	}
	$out .= '</data></array>';
	return (0,$out);
}
sub _mt_supportedtextfilters
{
	my $self = shift;
	return (2,undef) if (@_);
	my $out = '<array><data>';
	my @filters = sb::Plugin->get_text_filter;
	unshift(@filters,TEXT_FORMAT_1);
	foreach my $name (@filters)
	{
		my $key = ($name eq TEXT_FORMAT_1) ? 1 : $name;
		$out .= '<value><struct><member>' . "\n";
		$out .= '<name>label</name><value><string>' . $name . '</string></value>' . "\n";
		$out .= '</member><member>' . "\n";
		$out .= '<name>key</name><value><string>' . $key . '</string></value>' . "\n";
		$out .= '</member></struct></value>' . "\n";
	}
	$out .= '</data></array>';
	return (0,$out)
}
sub _getmediaobjectlist
{
	my $self = shift;
	my $wid  = shift; # string (not use)
	my $user = _value(shift); # string
	my $pass = _value(shift); # string
	my $num  = _value(shift); # int
	return (2,undef) if (@_);
	return (3,undef) if (!$self->check_login('user'=>$user,'pass'=>$pass));
	my @images = sb::Data->load('Image','bgn'=>0,'num'=>$num);
	my @out = ('<array><data>');
	foreach my $img (@images)
	{
		my $date = sb::Time->format('time'=>$img->date,'form'=>TIME_FORMAT);
		my $name = $img->name;
		push(@out,'<value><struct>');
		push(@out,'<member><name>objectid</name><value><string>' . $img->id . '</string></value></member>');
		push(@out,'<member><name>dateCreated</name><value><dateTime.iso8601>' . $date . '</dateTime.iso8601></value></member>');
		push(@out,'<member><name>name</name><value><string>' . $name . '</string></value></member>');
		push(@out,'<member><name>url</name><value><string>' . $img->get_url . '</string></value></member>');
		push(@out,'<member><name>type</name><value><string>' . $img->get_content_type . '</string></value></member>');
		push(@out,'</struct></value>');
	}
	push(@out,'</data></array>');
	return (0,join("\n",@out));
}
sub _getmediaobject
{
	my $self = shift;
	my $wid  = shift; # string (not use)
	my $iid  = _value(shift); # string
	my $user = _value(shift); # string
	my $pass = _value(shift); # string
	return (2,undef) if (@_);
	return (3,undef) if (!$self->check_login('user'=>$user,'pass'=>$pass));
	my $image = sb::Data->load('Image','id'=>$iid);
	return (4,undef) if (!$image);
	return (7,undef) if (!$self->has_permission('id'=>$image->auth));
	return (0,'<base64>' . $image->get_as_mime . '</base64>');
}
sub _deletemediaobject
{
	my $self = shift;
	my $wid  = shift; # string (not use)
	my $iid  = _value(shift); # string
	my $user = _value(shift); # string
	my $pass = _value(shift); # string
	return (2,undef) if (@_);
	return (3,undef) if (!$self->check_login('user'=>$user,'pass'=>$pass));
	return (5,undef) if (!$self->lock);
	my $image = sb::Data->load('Image','id'=>$iid);
	return (4,undef) if (!$image);
	return (7,undef) if (!$self->has_permission('id'=>$image->auth));
	$image->erase;
	sb::Data->update($image);
	return (0,'<boolean>1</boolean>');
}
# --------------------------------------
# // private functions
sub _template
{
	my $self = shift;
	my $err = shift;
	return <<'__ERROR_TEMPLATE__' if ($err);
<?xml version="1.0" encoding="{site_encoding}"?>
<methodResponse>
<fault><value><struct>
<member><name>faultCode</name><value><int>{xmlrpc_error}</int></value></member>
<member><name>faultString</name><value><string>{xmlrpc_message}</string></value></member>
</struct></value></fault>
</methodResponse>
__ERROR_TEMPLATE__
	return <<'__NORMAL_TEMPLATE__';
<?xml version="1.0" encoding="{site_encoding}"?>
<methodResponse>
<params><param><value>
{xmlrpc_message}
</value></param></params>
</methodResponse>
__NORMAL_TEMPLATE__
}
sub _method
{
	my $self = shift;
	return $self->cgi->value('methodName');
}
sub _params
{
	my $self = shift;
	return ($self->cgi->value('params') =~ m!<param>(.*?)</param>!sg);
}
sub _entry_struct
{
	my $self  = shift;
	my %param = (
		'entry' => undef,
		'mode'  => 'basic', # 'basic', 'simple', 'detail'
		@_
	);
	my @out = ();
	my $ent = $param{'entry'};
	return( undef ) if (!$ent);
	my $date = sb::Time->format('time'=>$ent->date,'form'=>TIME_FORMAT);
	push(@out,'<struct>');
	push(@out,'<member><name>userid</name><value><string>' . $ent->auth . '</string></value></member>');
	push(@out,'<member><name>dateCreated</name><value><dateTime.iso8601>' . $date . '</dateTime.iso8601></value></member>');
	push(@out,'<member><name>postid</name><value><string>' . $ent->id . '</string></value></member>');
	my %value_of = ();
	$value_of{'title'} = $ent->subj if ($param{'mode'} ne 'simple'); # 'basic' or 'detail'
	if ($self->has_permission('id'=>$ent->auth))
	{
		if ($param{'mode'} eq 'simple')
		{ # for [blogger.getRecentPosts]
			$value_of{'content'} = $ent->entitize('body');
		}
		elsif ($param{'mode'} eq 'detail')
		{
			$value_of{'description'} = $ent->entitize('body');
			$value_of{'mt_text_more'} = $ent->entitize('more');
			$value_of{'mt_keywords'} = $ent->key;
			$value_of{'mt_excerpt'} = ($ent->entitize('sum') ne '') ? $ent->sum : '';
			$value_of{'link'} = $ent->permalink('cat'=>$self->cat);
			$value_of{'permaLink'} = $ent->permalink('cat'=>$self->cat);
			$value_of{'mt_allow_comments'} = $ent->acm;
			$value_of{'mt_allow_pings'} = $ent->atb;
			$value_of{'mt_convert_breaks'} = $ent->form;
		}
	}
	else
	{
		$value_of{'sb_notAuthorized'} = 1;
	}
	foreach my $key ( keys(%value_of) )
	{
		my $type = ($key eq 'sb_notAuthorized') ? 'boolean' : 'string';
		push(@out,'<member><name>' . $key . '</name><value><' . $type . '>' . $value_of{$key} . '</' . $type . '></value></member>');
	}
	push(@out,'</struct>');
	return join("\n",@out);
}
sub _sort_categories
{
	my $self = shift;
	my %param = (
		'cat'    => [],
		'branch' => undef,   
		@_
	);
	my $list = '';
	foreach my $cat ( @{$param{'cat'}} )
	{
		my $parent = ($cat->main == -1) ? '' : $cat->main;
		next if (!defined($param{'branch'}) and $parent ne '');
		next if ( defined($param{'branch'}) and $parent ne $param{'branch'});
		$list .= $cat->id . ',';
		if ($cat->sub ne '')
		{
			$list .= $self->_sort_categories(
				'cat'    => $param{'cat'},
				'branch' => $cat->id,
			);
		}
	}
	return $list;
}
sub _build_files
{
	my $self = shift;
	my $entry = shift;
	return if (!$entry);
	my $type = $self->conf->value('conf_entry_archive');
	my $builder = sb::Build->new(
		'time'      => $self->time->data,
		'user'      => $self->users,
		'cat'       => $self->cat,
		'sortedcat' => [ sort { $b->order <=> $a->order } values(%{$self->cat}) ],
		'blog'      => $self->blog,
	);
	if ($type eq 'Individual')
	{
		my @entries = ($entry);
		my @ids = ($entry->id);
		my ($prv,$nxt) = $self->_get_neighbor($entry,'opened');
		push(@ids,$prv->id) if ($prv);
		push(@ids,$nxt->id) if ($nxt);
		@ids = sb::Utils->remove_duplicates(@ids);
		foreach my $id ( @ids )
		{
			next if ($id == $entry->id);
			my $tmp = sb::Data->load('Entry','id'=>$id);
			push(@entries,$tmp) if ($tmp);
		}
		foreach (@mScriptsForEntries)
		{ # generate javascripts first.
			$builder->build_javascript( $_ );
		};
		foreach my $ent ( @entries )
		{
			next if ($ent->stat == 0);
			$builder->build_entry( $ent );
		}
	}
	elsif ($type eq 'Monthly')
	{
		my $month = sb::Time->format(
			'time'=>$entry->date,
			'form'=>'%Year%%Mon%',
			'zone'=>$entry->tz
		);
		$builder->build_monthly_archive( $month );
	}
	$builder->build_category_index( $entry->cat ) if ($entry->cat ne '');
	$builder->set_latest_entries;
	$builder->build_top_page;
	$builder->build_feedfile('all');
}
sub _edit_entry
{
	my $self = shift;
	my %param = (
		'entry'   => undef,
		'content' => {},
		'open'    => 0,
		@_
	);
	my $entry = $param{'entry'};
	my $cont = $param{'content'};
	my %var = ();
	return if (!$self->user);
	# === preparation ===
	if ($self->lang->charset ne $self->charset)
	{
		my $charcode = $self->lang->code_for_charset($self->charset);
		foreach my $elem ('title','description','mt_text_more','mt_excerpt','mt_keywords')
		{
			$cont->{$elem} = $self->lang->convert($cont->{$elem},$self->lang->charcode,$charcode);
		}
	}
	foreach my $elem ('title','mt_keywords','mt_allow_comments','mt_allow_pings','mt_convert_breaks')
	{
		$cont->{$elem} =~ tr/\x0D\x0A//d; # removing linefeed
	}
	$cont->{'date'} = ($cont->{'dateCreated'})
		? $cont->{'dateCreated'}
		: $self->time;
	# === initilize variables ===
	$param{'open'} = 1 if ($self->conf->value('basic_xmlpublish'));
	$var{'elem'} = {
		'subj' => $cont->{'title'},
		'cat'  => $self->user->cat,
		'auth' => $self->user->id,
		'stat' => $param{'open'},
		'date' => $cont->{'date'}->data,
		'tz'   => $self->conf->value('conf_timezone'),
		'edit' => $self->user->id,
		'acm'  => ($cont->{'mt_allow_comments'} ne '') ? $cont->{'mt_allow_comments'} : $self->user->get_option('comment'),
		'atb'  => ($cont->{'mt_allow_pings'} ne '') ? $cont->{'mt_allow_pings'} : $self->user->get_option('trackback'),
		'form' => ($cont->{'mt_convert_breaks'} ne '') ? $cont->{'mt_convert_breaks'} : $self->user->get_option('format'),
		'body' => sb::Text->new($cont->{'description'})->detitized(),
		'more' => sb::Text->new($cont->{'mt_text_more'})->detitized(),
		'sum'  => $cont->{'mt_excerpt'},
		'key'  => $cont->{'mt_keywords'},
	};
	$var{'tbping'} = ($cont->{'mt_tb_ping_urls'}) ? $cont->{'mt_tb_ping_urls'} : [];
	$var{'ping'} = ($self->user->get_option('ping')) ? [split('\\n',$self->conf->value('conf_edit_ping'))] : [];
	if ($var{'elem'}->{'cat'} ne '' and $self->cat->{$var{'elem'}->{'cat'}})
	{
		$var{'elem'}->{'stat'} = 2 if ($var{'elem'}->{'stat'} and $self->cat->{$var{'elem'}->{'cat'}}->get_option('top'));
	}
	return if ( $var{'elem'}->{'body'} eq '' and $var{'elem'}->{'more'}  eq '' );
	# === update entry ===
	$entry = sb::Data->add('Entry') if (!$entry); # new entry
	if ($entry)
	{
		# copy data
		foreach my $elem ( keys(%{$var{'elem'}}) )
		{
			$entry->$elem($var{'elem'}->{$elem});
		}
		# send trackback ping
		if ($entry->stat and @{$var{'tbping'}})
		{
			my $stat = sb::Ping->new->send_trackback(
				'url'       => $entry->permalink,
				'excerpt'   => $entry->sum,
				'title'     => $entry->subj,
				'blog_name' => $self->blog->title,
				'list'      => $var{'tbping'},
				'eid'       => $entry->id,
				'now'       => $self->{'time'},
			);
			$entry->add_ping(@{$stat->{'sent'}});
			$entry->tmp(join("\n",@{$stat->{'error'}}));
		}
		# update data and static files
		sb::Data->update($entry);
		$self->_build_files($entry);
		# send update ping
		if ($entry->stat and @{$var{'ping'}})
		{
			my $stat = sb::Ping->new->send_update(
				'list' => $var{'ping'},
				'mode' => 'ping',
				'name' => $self->blog->title,
			);
		}
	}
	return $entry;
}
# --------------------------------------
# // private functions - for values
sub _type
{
	my $value = shift;
	$value =~ s!<value>(.*)</value>!$1!sg; # extract "value"
	$value =~ s!^[\s|\n]*(.*)[\s|\n]*$!$1!s; # extract "real" value
	TYPE_SWITCH: {
		$_ = $value;
		/^<int/ && do {
			return 'int' if ($value =~ m!<int>(.*)</int>!s);
			return 'int' if ($value =~ m!<int />!s);
			last TYPE_SWITCH;
		};
		/^<i4/ && do {
			return 'int' if ($value =~ m!<i4>(.*)</i4>!s);
			return 'int' if ($value =~ m!<int />!s);
			last TYPE_SWITCH;
		};
		/^<boolean/ && do {
			return 'bool' if ($value =~ m!<boolean>(.*)</boolean>!s);
			return 'bool' if ($value =~ m!<boolean />!s);
			last TYPE_SWITCH;
		};
		/^<double/ && do {
			return 'float' if ($value =~ m!<double>(.*)</double>!s);
			return 'float' if ($value =~ m!<double />!s);
			last TYPE_SWITCH;
		};
		/^<dateTime\.iso8601/ && do {
			return 'time' if ($value =~ m!<dateTime\.iso8601>(.*)</dateTime\.iso8601>!s);
			return 'time' if ($value =~ m!<dateTime\.iso8601 />!s);
			last TYPE_SWITCH;
		};
		/^<base64/ && do {
			return 'binary' if ($value =~ m!<base64>(.*)</base64>!s);
			return 'binary' if ($value =~ m!<base64 />!s);
			last TYPE_SWITCH;
		};
		/^<struct/ && do {
			return 'struct' if ($value =~ m!<struct>(.*)</struct>!s);
			return 'struct' if ($value =~ m!<struct />!s);
			last TYPE_SWITCH;
		};
		/^<array/ && do {
			return 'array' if ($value =~ m!<data>(.*)</data>!s);
			return 'array' if ($value =~ m!<array />!s);
			last TYPE_SWITCH;
		};
		/^<string/ && do {
			return 'string' if ($value =~ m!<string>(.*)</string>!s);
			return 'string' if ($value =~ m!<string />!s);
			last TYPE_SWITCH;
		};
	}
	return;
}
sub _value
{
	my $value = shift;
	$value =~ s!<value>(.*)</value>!$1!sg; # extract "value"
	$value =~ s!^[\s|\n]*(.*)[\s|\n]*$!$1!s; # extract "real" value
	TYPE_SWITCH: {
		$_ = $value;
		/^<int/ && do {
			return int($1) if ($value =~ m!<int>(.*)</int>!s);
			return if ($value =~ m!<int />!s);
			last TYPE_SWITCH;
		};
		/^<i4/ && do {
			return int($1) if ($value =~ m!<i4>(.*)</i4>!s);
			return if ($value =~ m!<int />!s);
			last TYPE_SWITCH;
		};
		/^<boolean/ && do {
			return int($1) if ($value =~ m!<boolean>(.*)</boolean>!s);
			return if ($value =~ m!<boolean />!s);
			last TYPE_SWITCH;
		};
		/^<double/ && do {
			return $1 if ($value =~ m!<double>(.*)</double>!s);
			return if ($value =~ m!<double />!s);
			last TYPE_SWITCH;
		};
		/^<dateTime\.iso8601/ && do {
			if ($value =~ m!<dateTime\.iso8601>(.*)</dateTime\.iso8601>!s)
			{
				my $str = $1;
				if ($str =~ /(\d\d\d\d)-?(\d\d)-?(\d\d)T(\d\d):(\d\d):(\d\d)/)
				{
					return sb::Time->new('year'=>$1,'mon'=>$2,'day'=>$3,'hour'=>$4,'min'=>$5,'sec'=>$6);
				}
				return;
			}
			return if ($value =~ m!<dateTime\.iso8601 />!s);
			last TYPE_SWITCH;
		};
		/^<base64/ && do {
			if ($value =~ m!<base64>(.*)</base64>!s)
			{
				require 'mimeutil.pl';
				my $output = undef;
				my $buf = $1;
				$buf =~ s/\s//g; # by KIKUCHI Kouji
				$buf =~ s/(.{76})/$1\n/g;
				$output  = &mimeutil::bodydecode($buf,'b64');
				$output .= &mimeutil::bdeflush('b64');
				return $output;
			}
			return if ($value =~ m!<base64 />!s);
			last TYPE_SWITCH;
		};
		/^<struct/ && do {
			return _struct($1) if ($value =~ m!<struct>(.*)</struct>!s);
			return {} if ($value =~ m!<struct />!s);
			last TYPE_SWITCH;
		};
		/^<array/ && do {
			return _array($1) if ($value =~ m!<data>(.*)</data>!s);
			return [] if ($value =~ m!<array />!s);
			last TYPE_SWITCH;
		};
		/^<string/ && do {
			return $1 if ($value =~ m!<string>(.*)</string>!s);
			return '' if ($value =~ m!<string />!s);
			last TYPE_SWITCH;
		};
	}
	return;
}
sub _struct
{
	my $input = shift;
	my $output;
	my @stack = ($input =~ m!<member>(.*?)</member>!sg); # imperfect
	foreach my $member (@stack)
	{
		if ($member =~ m!<name>(.*?)</name>.*<value>(.*?)</value>!s)
		{
			$output->{$1} = _value($2);
		}
	}
	return $output;
}
sub _array
{
	my $input = shift;
	my $output;
	my $flag = 0;
	if ($input =~ m!<value>[\s|\n]*<struct>!)
	{ # struct in array ?
		my @stack = ($input =~ m!<value>[\s|\n]*<struct>(.*?)</struct>[\s|\n]*</value>!sg); # imperfect
		foreach my $value (@stack)
		{
			push(@{$output},_struct($value));
		}
	}
	else
	{ # probably just an array
		my @stack = ($input =~ m!<value>(.*?)</value>!sg); # imperfect
		foreach my $value (@stack)
		{
			push(@{$output},_value($value));
		}
	}
	return $output;
}
1;
__END__
